@@ -172,12 +172,12 @@ function task_termini {
     install -Dm 0644 puppet/util/puppetdb/config.rb "${DESTDIR}${rubylibdir}/puppet/util/puppetdb/config.rb"
     install -Dm 0644 puppet/util/puppetdb/http.rb "${DESTDIR}${rubylibdir}/puppet/util/puppetdb/http.rb"
     install -Dm 0644 puppet/util/puppetdb/char_encoding.rb "${DESTDIR}${rubylibdir}/puppet/util/puppetdb/char_encoding.rb"
+    install -Dm 0644 puppet/util/puppetdb/atom.rb "${DESTDIR}${rubylibdir}/puppet/util/puppetdb/atom.rb"
     install -Dm 0644 puppet/util/puppetdb.rb "${DESTDIR}${rubylibdir}/puppet/util/puppetdb.rb"
-    install -Dm 0644 puppet/application/storeconfigs.rb "${DESTDIR}${rubylibdir}/puppet/application/storeconfigs.rb"
     install -Dm 0644 puppet/reports/puppetdb.rb "${DESTDIR}${rubylibdir}/puppet/reports/puppetdb.rb"
     install -Dm 0644 puppet/face/node/deactivate.rb "${DESTDIR}${rubylibdir}/puppet/face/node/deactivate.rb"
     install -Dm 0644 puppet/face/node/status.rb "${DESTDIR}${rubylibdir}/puppet/face/node/status.rb"
-    install -Dm 0644 puppet/face/storeconfigs.rb "${DESTDIR}${rubylibdir}/puppet/face/storeconfigs.rb"
+    install -Dm 0644 puppet/functions/puppetdb_query.rb "${DESTDIR}${rubylibdir}/puppet/functions/puppetdb_query.rb"
 }
 
 function task_defaults {
