@@ -1,8 +1,8 @@
-(def tk-version "1.1.1")
-(def ks-version "1.0.0")
+(def tk-version "1.3.1")
+(def ks-version "1.3.0")
 (def jetty-version "9.2.10.v20150310")
 
-(defproject puppetlabs/trapperkeeper-webserver-jetty9 "1.5.0"
+(defproject puppetlabs/trapperkeeper-webserver-jetty9 "1.5.8"
   :description "A jetty9-based webserver implementation for use with the puppetlabs/trapperkeeper service framework."
   :url "https://github.com/puppetlabs/trapperkeeper-webserver-jetty9"
   :license {:name "Apache License, Version 2.0"
@@ -11,17 +11,20 @@
   ;; dependencies. Also supports :warn to simply emit warnings.
   ;; requires lein 2.2.0+.
   :pedantic? :abort
-  :dependencies [[org.clojure/clojure "1.6.0"]
+  :dependencies [[org.clojure/clojure "1.7.0"]
+
+                 ;; begin version conflict resolution dependencies
+                 ;; end version conflict resolution dependencies
+
+                 [org.clojure/java.jmx "0.3.1"]
                  [org.clojure/tools.logging "0.2.6"]
-                 [prismatic/schema "0.4.0"]
-                 [prismatic/plumbing "0.4.2"]
+                 [prismatic/schema "1.0.4"]
 
-                 ;; Let ssl-utils bring in the clj-time dependency
                  [puppetlabs/ssl-utils "0.8.1"]
-                 [puppetlabs/kitchensink ~ks-version :exclusions [clj-time]]
-                 [puppetlabs/trapperkeeper ~tk-version :exclusions [clj-time]]
+                 [puppetlabs/kitchensink ~ks-version]
+                 [puppetlabs/trapperkeeper ~tk-version]
 
-                 [ch.qos.logback/logback-access "1.1.1"]
+                 [ch.qos.logback/logback-access "1.1.3"]
 
                  [org.codehaus.janino/janino "2.7.8"]
 
@@ -39,6 +42,9 @@
 
                  [ring/ring-servlet "1.1.8" :exclusions [javax.servlet/servlet-api commons-codec]]]
 
+  :source-paths  ["src"]
+  :java-source-paths  ["java"]
+
   :plugins [[lein-release "1.0.5" :exclusions [org.clojure/clojure]]]
 
   :lein-release {:scm         :git
@@ -64,11 +70,10 @@
                                   "examples/webrouting_app/src"]
                    :java-source-paths ["examples/servlet_app/src/java"
                                        "test/java"]
-                   :dependencies [[puppetlabs/http-client "0.4.4"]
+                   :dependencies [[puppetlabs/http-client "0.5.0"]
                                   [puppetlabs/kitchensink ~ks-version :classifier "test"]
                                   [puppetlabs/trapperkeeper ~tk-version :classifier "test"]
                                   [org.clojure/tools.namespace "0.2.10"]
-                                  [org.clojure/java.jmx "0.2.0"]
                                   [spyscope "0.1.4"]
                                   [compojure "1.1.8" :exclusions [ring/ring-core
                                                                   commons-io
@@ -83,7 +88,8 @@
              :testutils {:source-paths ^:replace ["test/clj"]
                          :java-source-paths ^:replace ["test/java"]}
              :uberjar {:aot [puppetlabs.trapperkeeper.main]
-                       :classifiers ^:replace []}}
+                       :classifiers ^:replace []
+                       :dependencies [[org.clojure/tools.nrepl "0.2.6" :scope "test" :exclusions [[org.clojure/clojure]]]]}}
 
   :main puppetlabs.trapperkeeper.main
   )
