@@ -1,4 +1,4 @@
-(def pdb-version "4.0.2")
+(def pdb-version "4.1.0")
 
 (defn deploy-info
   "Generate deployment information from the URL supplied and the username and
@@ -9,10 +9,11 @@
    :password :env/nexus_jenkins_password
    :sign-releases false})
 
-(def tk-version "1.2.0")
-(def tk-jetty9-version "1.5.0")
-(def ks-version "1.2.0")
+(def tk-version "1.3.1")
+(def tk-jetty9-version "1.5.8")
+(def ks-version "1.3.1")
 (def tk-status-version "0.3.1")
+(def i18n-version "0.2.2")
 
 (def pdb-jvm-opts
   (case (System/getProperty "java.specification.version")
@@ -33,7 +34,8 @@
   :pedantic? :abort
 
   :dependencies [[org.clojure/clojure "1.8.0"]
-                 [cheshire "5.5.0"]
+                 [puppetlabs/i18n ~i18n-version]
+                 [cheshire "5.6.1"]
                  [org.clojure/core.match "0.3.0-alpha4" :exclusions [org.clojure/tools.analyzer.jvm]]
                  [org.clojure/math.combinatorics "0.1.1"]
                  [org.clojure/math.numeric-tower "0.0.4"]
@@ -43,9 +45,8 @@
                  [clj-stacktrace "0.2.8"]
                  [metrics-clojure "2.6.1" :exclusions [org.clojure/clojure org.slf4j/slf4j-api]]
                  [clj-time "0.11.0"]
-                 [org.clojure/java.jmx "0.3.1"]
                  ;; Filesystem utilities
-                 [me.raynes/fs "1.4.5"]
+                 [me.raynes/fs "1.4.6"]
                  [org.apache.commons/commons-lang3 "3.3.1"]
                  ;; Version information
                  [puppetlabs/dujour-version-check "0.1.3"]
@@ -57,7 +58,7 @@
                  ;; Database connectivity
                  [com.zaxxer/HikariCP "2.4.3"]
                  [org.clojure/java.jdbc "0.4.2"]
-                 [org.postgresql/postgresql "9.4-1206-jdbc4"]
+                 [org.postgresql/postgresql "9.4.1208.jre7"]
 
                  ;; MQ connectivity
                  [org.apache.activemq/activemq-broker "5.13.2" :exclusions [org.slf4j/slf4j-api]]
@@ -70,13 +71,14 @@
                  ;; bridge to allow some spring/activemq stuff to log over slf4j
                  [org.slf4j/jcl-over-slf4j "1.7.14" :exclusions [org.slf4j/slf4j-api]]
                  ;; WebAPI support libraries.
-                 [compojure "1.4.0"]
+                 [compojure "1.5.0"]
                  [clj-http "2.0.1"]
                  [ring/ring-core "1.4.0" :exclusions [javax.servlet/servlet-api org.clojure/tools.reader]]
                  [org.apache.commons/commons-compress "1.10"]
                  [puppetlabs/kitchensink ~ks-version]
                  [puppetlabs/trapperkeeper ~tk-version]
                  [puppetlabs/trapperkeeper-webserver-jetty9 ~tk-jetty9-version]
+                 [puppetlabs/trapperkeeper-metrics "0.2.0"]
                  [prismatic/schema "1.0.4"]
                  [trptcolin/versioneer "0.2.0"]
                  [puppetlabs/trapperkeeper-status ~tk-status-version]
@@ -99,8 +101,9 @@
   :repositories [["releases" "http://nexus.delivery.puppetlabs.net/content/repositories/releases/"]
                  ["snapshots" "http://nexus.delivery.puppetlabs.net/content/repositories/snapshots/"]]
 
-  :plugins [[lein-release "1.0.5"]
-            [lein-cloverage "1.0.6" :exclusions [org.clojure/clojure]]]
+  :plugins [[lein-release "1.0.5" :exclusions [org.clojure/clojure]]
+            [lein-cloverage "1.0.6" :exclusions [org.clojure/clojure]]
+            [puppetlabs/i18n ~i18n-version]]
 
   :lein-release {:scm        :git
                  :deploy-via :lein-deploy}
@@ -139,7 +142,7 @@
              :ezbake {:dependencies ^:replace [[puppetlabs/puppetdb ~pdb-version]
                                                [org.clojure/tools.nrepl "0.2.3"]]
                       :name "puppetdb"
-                      :plugins [[puppetlabs/lein-ezbake "0.3.24"
+                      :plugins [[puppetlabs/lein-ezbake "0.3.25"
                                  :exclusions [org.clojure/clojure]]]}
              :testutils {:source-paths ^:replace ["test"]}
              :ci {:plugins [[lein-pprint "1.1.1"]]}}
