@@ -44,11 +44,10 @@
      maintain acceptable performance."
   (:require [clj-time.core :refer [ago]]
             [clojure.java.io :as io]
-            [clojure.java.jmx :as jmx]
             [clojure.tools.logging :as log]
             [compojure.core :as compojure]
             [metrics.reporters.jmx :as jmx-reporter]
-            [overtone.at-at :refer [mk-pool interspaced]]
+            [overtone.at-at :refer [mk-pool interspaced stop-and-reset-pool!]]
             [puppetlabs.kitchensink.core :as kitchensink]
             [puppetlabs.puppetdb.cheshire :as json]
             [puppetlabs.puppetdb.command.constants :refer [command-names]]
@@ -92,9 +91,8 @@
      (format "sweep of stale nodes (threshold: %s)"
              (format-period node-ttl))
      (jdbc/with-transacted-connection db
-       (doseq [node (scf-store/stale-nodes (ago node-ttl))]
-         (log/infof "Auto-expiring node %s" node)
-         (scf-store/expire-node! node))))
+       (doseq [node (scf-store/expire-stale-nodes node-ttl)]
+         (log/infof "Auto-expired node %s" node))))
     (catch Exception e
       (log/error e "Error while deactivating stale nodes"))))
 
@@ -176,6 +174,8 @@
     (.close ds))
   (when-let [ds (get-in context [:shared-globals :scf-read-db :datasource])]
     (.close ds))
+  (when-let [pool (:job-pool context)]
+    (stop-and-reset-pool! pool))
   context)
 
 (defn- transfer-old-messages! [mq-endpoint]
@@ -256,8 +256,7 @@
     (when-let [v (version/version)]
       (log/infof "PuppetDB version %s" v))
 
-    (when (init-with-db database config)
-      (sutils/async-vacuum-analyze write-db))
+    (init-with-db database config)
 
     (let [population-registry (get-in metrics/metrics-registries [:population :registry])]
       (pop/initialize-population-metrics! population-registry read-db))
@@ -304,10 +303,11 @@
         (when (pos? dlo-compression-interval-millis)
           (interspaced dlo-compression-interval-millis
                        #(compress-dlo! dlo-compression-threshold discard-dir)
-                       job-pool)))
-      (assoc context
-             :broker broker
-             :shared-globals globals))))
+                       job-pool))
+        (assoc context
+               :job-pool job-pool
+               :broker broker
+               :shared-globals globals)))))
 
 (defprotocol PuppetDBServer
   (shared-globals [this])
