@@ -439,6 +439,35 @@
     "ALTER TABLE ONLY resource_params
         ADD CONSTRAINT resource_params_resource_fkey FOREIGN KEY (resource) REFERENCES resource_params_cache(resource) ON DELETE CASCADE"))
 
+(def add-factset-id-fk-constraint-cmd
+  "alter table facts add constraint factset_id_fk
+     foreign key (factset_id) references factsets(id)
+     on update cascade on delete cascade")
+
+(def add-resource-events-report-id-fkey-constraint-cmd
+  "alter table resource_events add constraint resource_events_report_id_fkey
+     foreign key (report_id) references reports(id) on delete cascade")
+
+(def add-catalogs-certname-fkey-constraint-cmd
+  "alter table catalogs add constraint catalogs_certname_fkey
+     foreign key (certname) references certnames(certname)
+     on update no action on delete cascade")
+
+(def add-factsets-certname-fk-constraint-cmd
+  "alter table factsets add constraint factsets_certname_fk
+     foreign key (certname) references certnames(certname)
+     on update cascade on delete cascade")
+
+(def add-reports-certname-fk-constraint-cmd
+  "alter table reports add constraint reports_certname_fkey
+     foreign key (certname) references certnames(certname)
+     on delete cascade")
+
+(def add-certnames-reports-id-fkey-constraint-cmd
+  "alter table certnames add constraint certnames_reports_id_fkey
+     foreign key (latest_report_id) references reports(id)
+     on delete set null")
+
 (defn version-2yz-to-300-migration
   ;; This migration includes:
   ;;   Insertion of the factsets hash column
@@ -736,17 +765,17 @@
       "ALTER TABLE factsets ADD CONSTRAINT factsets_environment_id_fk
        FOREIGN KEY (environment_id) REFERENCES environments(id)
        ON UPDATE RESTRICT ON DELETE RESTRICT"
-      "ALTER TABLE facts ADD CONSTRAINT factset_id_fk
-       FOREIGN KEY (factset_id) REFERENCES factsets(id)
-       ON UPDATE CASCADE ON DELETE CASCADE"
+
+      add-factset-id-fk-constraint-cmd
+
       "ALTER TABLE factsets ADD CONSTRAINT factsets_certname_idx
          UNIQUE (certname)"
       "ALTER TABLE factsets ADD CONSTRAINT factsets_hash_key UNIQUE (hash)"
 
       "ALTER TABLE resource_events ADD CONSTRAINT resource_events_unique
          UNIQUE (report_id, resource_type, resource_title, property)"
-      "ALTER TABLE resource_events ADD CONSTRAINT resource_events_report_id_fkey
-         FOREIGN KEY (report_id) REFERENCES reports(id) ON DELETE CASCADE"
+
+      add-resource-events-report-id-fkey-constraint-cmd
 
       "ALTER TABLE certnames ADD CONSTRAINT certnames_pkey
          PRIMARY KEY (certname)"
@@ -755,19 +784,11 @@
       "ALTER TABLE edges ADD CONSTRAINT edges_certname_fkey
          FOREIGN KEY (certname) REFERENCES certnames(certname)
          ON UPDATE NO ACTION ON DELETE CASCADE"
-      "ALTER TABLE catalogs ADD CONSTRAINT catalogs_certname_fkey
-         FOREIGN KEY (certname) REFERENCES certnames(certname)
-         ON UPDATE NO ACTION ON DELETE CASCADE"
-      "ALTER TABLE factsets ADD CONSTRAINT factsets_certname_fk
-         FOREIGN KEY (certname) REFERENCES certnames(certname)
-         ON UPDATE CASCADE ON DELETE CASCADE"
-      "ALTER TABLE reports ADD CONSTRAINT reports_certname_fkey
-         FOREIGN KEY (certname) REFERENCES certnames(certname)
-         ON DELETE CASCADE"
 
-      "ALTER TABLE certnames ADD CONSTRAINT certnames_reports_id_fkey
-         FOREIGN KEY (latest_report_id) REFERENCES reports(id)
-         ON DELETE SET NULL")))
+      add-catalogs-certname-fkey-constraint-cmd
+      add-factsets-certname-fk-constraint-cmd
+      add-reports-certname-fk-constraint-cmd
+      add-certnames-reports-id-fkey-constraint-cmd)))
 
 (defn add-expired-to-certnames
   "Add a 'expired' column to the 'certnames' table, to track
@@ -860,8 +881,14 @@
    "ALTER TABLE catalog_resources RENAME TO catalog_resources_tmp"
    ;; CREATE certnames and catalog_resources transform tables
    "CREATE TABLE catalog_resources (LIKE catalog_resources_tmp INCLUDING ALL)"
+   ;; Make sure to update the constraints in
+   ;; purge-deactivated-and-expired-nodes-new! if you change
+   ;; the columns here.
    "CREATE TABLE latest_catalogs (catalog_id BIGINT NOT NULL UNIQUE REFERENCES catalogs(id) ON DELETE CASCADE, certname_id BIGINT PRIMARY KEY REFERENCES certnames(id) ON DELETE CASCADE)"
    "ALTER TABLE catalog_resources DROP COLUMN catalog_id"
+   ;; Make sure to update the constraint in
+   ;; purge-deactivated-and-expired-nodes-new! if you change
+   ;; the columns here.
    "ALTER TABLE catalog_resources ADD COLUMN certname_id BIGINT NOT NULL REFERENCES certnames(id) ON DELETE CASCADE"
    "ALTER TABLE catalog_resources ADD PRIMARY KEY (certname_id, type, title)"
 
@@ -951,19 +978,11 @@
    "ALTER TABLE certnames DROP CONSTRAINT certnames_pkey CASCADE"
    "DROP TABLE certnames"
    "ALTER TABLE certnames_transform RENAME to certnames"
-   "ALTER TABLE catalogs ADD CONSTRAINT catalogs_certname_fkey
-     FOREIGN KEY (certname) REFERENCES certnames(certname)
-     ON UPDATE NO ACTION ON DELETE CASCADE"
-   "ALTER TABLE factsets ADD CONSTRAINT factsets_certname_fk
-     FOREIGN KEY (certname) REFERENCES certnames(certname)
-     ON UPDATE CASCADE ON DELETE CASCADE"
-   "ALTER TABLE reports ADD CONSTRAINT reports_certname_fkey
-     FOREIGN KEY (certname)
-     REFERENCES certnames(certname) ON DELETE CASCADE"
 
-   "ALTER TABLE certnames ADD CONSTRAINT certnames_reports_id_fkey
-      FOREIGN KEY (latest_report_id)
-      REFERENCES reports(id) ON DELETE SET NULL"))
+   add-catalogs-certname-fkey-constraint-cmd
+   add-factsets-certname-fk-constraint-cmd
+   add-reports-certname-fk-constraint-cmd
+   add-certnames-reports-id-fkey-constraint-cmd))
 
 (defn add-certname-id-to-resource-events
   []
@@ -1013,8 +1032,8 @@
      ON resource_events(resource_title)"
    "CREATE INDEX resource_events_status_idx ON resource_events(status)"
    "CREATE INDEX resource_events_timestamp_idx ON resource_events(timestamp)"
-   "ALTER TABLE resource_events ADD CONSTRAINT resource_events_report_id_fkey
-     FOREIGN KEY (report_id) REFERENCES reports(id) ON DELETE CASCADE"))
+
+   add-resource-events-report-id-fkey-constraint-cmd))
 
 (defn add-catalog-uuid-to-reports-and-catalogs
   []
@@ -1024,6 +1043,11 @@
    "UPDATE catalogs SET catalog_uuid=catalogs.transaction_uuid WHERE hash is NULL"
    "UPDATE reports SET catalog_uuid=reports.transaction_uuid WHERE hash is NULL"))
 
+(defn index-certnames-latest-report-id
+  []
+  (jdbc/do-commands
+    "CREATE INDEX idx_certnames_latest_report_id on certnames(latest_report_id)"))
+
 (def migrations
   "The available migrations, as a map from migration version to migration function."
   {28 init-through-2-3-8
@@ -1045,7 +1069,8 @@
    41 factset-hash-field-not-nullable
    42 add-support-for-historical-catalogs
    43 add-indexes-for-reports-summary-query
-   44 add-catalog-uuid-to-reports-and-catalogs})
+   44 add-catalog-uuid-to-reports-and-catalogs
+   45 index-certnames-latest-report-id})
 
 (def desired-schema-version (apply max (keys migrations)))
 
@@ -1132,7 +1157,8 @@
   [db-connection-pool]
   (let [applied-migration-versions (applied-migrations)
         latest-applied-migration (last applied-migration-versions)
-        known-migrations (apply sorted-set (keys migrations))]
+        known-migrations (apply sorted-set (keys migrations))
+        small-tables ["value_types" "report_statuses"]]
 
     (when (and latest-applied-migration
                (< latest-applied-migration (first known-migrations)))
@@ -1153,8 +1179,8 @@
           (doseq [[version migration] pending]
             (log/infof "Applying database migration version %d" version)
             (sql-or-die (fn [] (migration) (record-migration! version)))))
-        true
-        )
+        (sutils/analyze-small-tables small-tables)
+        true)
       (do
         (log/info "There are no pending migrations")
         false))))
